
import datetime
import os
import urllib

import gtk

from gimmie_base import ItemSource
from gimmie_file import FileItem
from gimmie_util import recent_model


class RecentlyUsed(ItemSource):
    '''
    Recently-used documents, log stored in ~/.recently-used.
    '''
    def __init__(self, name, icon = "stock_calendar"):
        ItemSource.__init__(self, name, icon)
        
        recent_model.connect("changed", self._model_changed)

    def _model_changed(self, model = None, list = None):
        self.emit("reload")

    def do_reload(self):
        recent_items = []

        for item in recent_model.get_list():
            # skip private items (like porn)
            if item.get_private():
                continue
            
            # skip deleted local files
            scheme, path = urllib.splittype(item.get_uri())
            if scheme == "file" and not os.path.exists(urllib.url2pathname(path)):
                continue

            # check whether to include this item
            if self.include_recent_item(item):
                recent_item = FileItem(item.get_uri(), item.get_timestamp())
                recent_items.append(recent_item)

        ### Uncomment to sort by most recently used
        #recent_items.sort(lambda x, y: y.get_timestamp() - x.get_timestamp())

        # Sort alphabetically
        recent_items.sort(key=lambda o: o.get_name().lower())

        self.set_items(recent_items)

    def include_recent_item(self, item):
        return True


class RecentlyUsedOfMimeType(RecentlyUsed):
    '''
    Recently-used items filtered by a set of mimetypes.
    '''
    def __init__(self, name, icon, mimetype_list):
        RecentlyUsed.__init__(self, name, icon)
        self.mimetype_list = mimetype_list

    def include_recent_item(self, item):
        return item.get_mime_type() in self.mimetype_list

